;;;; avrforth
;;;;
;;;; public domain


.include "target/defines"

.listmac

;;; user defines

  .set ramend = $45f

  .set return_stack_size = $80

  .set upinit = $200

  .set cpu_frequency = 3860000
  .set baud_rate = 19200
  ;.set baud_rate = 31250

  
;;; register defines

  ; z
  .def zh = r31
  .def zl = r30

  ; y : parameter stack pointer
  .def yh = r29
  .def yl = r28

  ; x
  .def xh = r27
  .def xl = r26

  ; top of stack register
  .def tosh = r25
  .def tosl = r24
  .def tos = r24
  
  ; temp registers
  .def temp3 = r19
  .def temp2 = r18
  .def temp1 = r17
  .def temp0 = r16

  ; zero register
  .def zeroh = r15
  .def zerol = r14
  .def zero = r14

  ; saved return address
  ; for context switches

  .def srh = r5
  .def srl = r4
  .def sr = r4

  ; user pointer
  .def runningh = r3
  .def runningl = r2
  .def running = r2


.org $0


;;;; vectors

.include "target/vectors"


;;;; interrupt revector tables

.include "int.asm"

;;;; words

.set link = $0
.set mlink = $0

;;; stack manipulations

.include "words/dup.asm" ;;;
.include "words/drop.asm" ;;;
.include "words/nip.asm" ;;;
.include "words/over.asm"
.include "words/rot.asm" ;;;
.include "words/swap.asm" ;;;

.include "words/spstore.asm" ;;;
.include "words/spfetch.asm" ;;;

;;; return stack manipulations

.include "words/tor.asm" ;;;
.include "words/rfrom.asm" ;;;

.include "words/rpstore.asm" ;;;
.include "words/rpfetch.asm" ;;;

;;; constants

.include "const.asm" ;;;

;;; user area variables

.include "user.asm" ;;;

;;; memory operators

.include "words/store.asm" ;;;
.include "words/cstore.asm" ;;;
.include "words/fetch.asm" ;;;
.include "words/cfetch.asm" ;;;
.include "words/ifetch.asm"
.include "words/efetch.asm" ;;;

;;; math

.include "words/twostar.asm" ;;;
.include "words/twoslash.asm" ;;;
.include "words/minus.asm" ;;;
.include "words/plus.asm" ;;;
.include "words/plusstore.asm" ;;;

.include "words/oneminus.asm"
.include "words/oneplus.asm"

.include "words/umin.asm" ;;;
.include "words/umax.asm" ;;;

;;; logical

.include "words/and.asm" ;;;
.include "words/or.asm" ;;;
.include "words/swapbytes.asm"
.include "words/xor.asm" ;;;

;;; interpreter

.include "words/accept.asm" ;;;
.include "words/execute.asm"
.include "words/fetchexecute.asm" ;;;
.include "words/find.asm" ;;;
.include "words/lit.asm"
.include "words/word.asm" ;;;
.include "words/go.asm" ;;;
.include "words/tick.asm" ;;;
.include "words/forth.asm" ;;;
.include "words/macro.asm" ;;;

;;; compiler

.include "words/colon.asm" ;;;
.include "words/rightbracket.asm" ;;;
.include "words/c.asm" ;;;
.include "words/semicolon.asm" ;;;
.include "words/doublesemicolon.asm" ;;;
.include "words/l.asm" ;;;
.include "words/commaxt.asm" ;;;

;;; dictionary

.include "words/allot.asm" ;;;
.include "words/iallot.asm" ;;;
.include "words/here.asm" ;;;
.include "words/ihere.asm" ;;;
.include "words/comma.asm" ;;;
.include "words/icomma.asm" ;;;
.include "words/latest.asm" ;;;

;;; i/o

.include "words/cr.asm" ;;;
.include "words/space.asm" ;;;
.include "words/type.asm" ;;;
.include "words/itype.asm" ;;;
.include "words/xquote.asm" ;;;
.include "words/udot.asm" ;;;

;;; numeric input

.include "num.asm" ;;;

;;; alignment and portability

.include "words/cells.asm" ;;;
.include "words/icells.asm" ;;;

;;; control

.include "words/if.asm" ;;;
.include "words/then.asm" ;;;

;;; startup

.include "words/cold.asm" ;;;

;;; uart

.include "words/emit.asm"
.include "words/key.asm"


;;; main entry point

reset:

  ; init return stack pointer
  ldi temp0,high(ramend)
  out SPH,temp0
  ldi temp0,low(ramend)
  out SPL,temp0

  ; init parameter stack pointer
  ldi yh,high(ramend-return_stack_size+1)
  ldi yl,low(ramend-return_stack_size+1)

  ; init user area pointer
  ldi temp1,high(upinit)
  ldi temp0,low(upinit)
  movw running,temp0

  ; set baud rate of uart
  ldi temp0,high((cpu_frequency/(baud_rate*16))-1)
  out UBRRH,temp0
  ldi temp0,low((cpu_frequency/(baud_rate*16))-1)
  out UBRRL,temp0

  ; enable transmitter and receiver
  ldi temp0,(1<<TXEN)|(1<<RXEN)
  out UCSRB,temp0

  ; init zeroes register
  clr zeroh
  clr zerol

  ; setup timer0
  ldi temp0,(1<<CS01)|(1<<CS00)
  out TCCR0,temp0

  ldi temp0,(1<<TOIE0)
  out TIMSK,temp0

  ; make portc all outputs and clear it
  ; stk500 leds light when 1 is output to pin
  ldi temp0,$ff
  out $14,temp0

  out $15,temp0

  rjmp cold


;; context!

.dw link
.set link = pc - 1
.db 'c','o','n','t','e','x','t','!',0

contextstore:
  pop srh
  pop srl
  push r31
  push r30
  push r29
  push r28
  push r27
  push r26
  push r25
  push r24
  push r23
  push r22
  push r21
  push r20
  push r19
  push r18
  push r17
  push r16
  push r1
  push r0
  in r0,SREG
  push r0

  rcall rpfetch
  rcall runningfetch
  rcall store

  movw z,sr
  ijmp
  
;  push srl
;  push srh
;  ret


;; inittask - work in progress!!!
;;
;; ( xt addr -- )

.dw link
.set link = pc - 1
.db 'i','n','i','t','t','a','s','k',0

inittask:
  ; xt in x
  ld xl,y+
  ld xh,y+

  movw z,tos
  subi zl,-$80
  sbci zh,-$00

  movw temp0,tos
  subi temp0,-$00
  sbci temp1,-$01

  push xl
  push xh
  push zero
  push zero

  ; psp
  push temp1
  push temp0
  
  push zero
  push zero
  push zero
  push zero
  push zero
  push zero
  push zero
  push zero
  push zero
  push zero
  push zero
  push zero
  push zero
  push zero

  ; sreg
  ldi temp2,$80
  push temp2

  ; rp points to next available
  ; address registers point one higher
  adiw z,1
  dup_
  movw tos,zl
  
;  rcall upfetch
;  rcall rot
;  rcall upstore
;  rcall swapp
;  rcall context
;  rcall store
;  rcall upstore
  ret


;; context@

.dw link
.set link = pc - 1
.db 'c','o','n','t','e','x','t','@',0

contextfetch:
  ; get frame pointer
  rcall runningfetch
  rcall fetch
  rcall rpstore

  ; restore registers
  pop r0
  out SREG,r0
  pop r0
  pop r1
  pop r16
  pop r17
  pop r18
  pop r19
  pop r20
  pop r21
  pop r22
  pop r23
  pop r24
  pop r25
  pop r26
  pop r27
  pop r28
  pop r29
  pop r30
  pop r31
  reti


codemem:


.org $0c00

.include "words/istore.asm"

.set lastword = link
.set lastmacro = mlink

